package it.unimi.dsi.big.webgraph.algo;

import static org.junit.Assert.assertEquals;
import it.unimi.dsi.big.webgraph.ImmutableGraph;
import it.unimi.dsi.bits.LongArrayBitVector;
import it.unimi.dsi.fastutil.longs.LongBigArrays;
import it.unimi.dsi.webgraph.ArrayListMutableGraph;

import org.junit.Test;

public class StronglyConnectedComponentsTest {

	@Test
	public void testBuckets() {
		final ImmutableGraph g = ImmutableGraph.wrap( new ArrayListMutableGraph( 9, 
				new int[][] { { 0, 0 }, { 1, 0 }, { 1, 2 }, 
				{ 2, 1 }, { 2, 3 }, { 2, 4 }, { 2, 5 },
				{ 3, 4 }, { 4, 3 },
				{ 5, 5 }, { 5, 6 }, { 5, 7 }, { 5, 8 },
				{ 6, 7 },
				{ 8, 7 } }
		).immutableView() );

		StronglyConnectedComponents components = StronglyConnectedComponents.compute( g, true, null );

		LongArrayBitVector buckets = LongArrayBitVector.ofLength( g.numNodes() );
		buckets.set( 0, true );
		buckets.set( 3, true );
		buckets.set( 4, true );
		assertEquals( buckets, components.buckets );
		assertEquals( 3, buckets.count() );
		
		final long[][] size = components.computeSizes();
		components.sortBySize( size );
		
		assertEquals( 2, LongBigArrays.get( size, 0 ) );
		assertEquals( 2, LongBigArrays.get( size, 1 ) );
		assertEquals( 1, LongBigArrays.get( size, 2 ) );
		assertEquals( 1, LongBigArrays.get( size, 3 ) );
		assertEquals( 1, LongBigArrays.get( size, 4 ) );
		assertEquals( 1, LongBigArrays.get( size, 5 ) );
		assertEquals( 1, LongBigArrays.get( size, 6 ) );		
	}

	@Test
	public void testCompleteGraph() {
		StronglyConnectedComponents components = StronglyConnectedComponents.compute( ImmutableGraph.wrap( ArrayListMutableGraph.newCompleteGraph( 5, false ).immutableView() ), true, null );
		assertEquals( 5, components.buckets.count() );
		for( int i = 5; i-- != 0; ) assertEquals( 0, LongBigArrays.get( components.component, i ) );
		assertEquals( 5, components.computeSizes()[ 0 ][ 0 ] );
	}

	@Test
	public void testTree() {
		StronglyConnectedComponents components = StronglyConnectedComponents.compute( ImmutableGraph.wrap( ArrayListMutableGraph.newCompleteBinaryIntree( 3 ).immutableView() ), true, null );
		assertEquals( 0, components.buckets.count() );
		assertEquals( 15, components.numberOfComponents );
	}
}

